<?php

namespace Drupal\gift_aid_commerce\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\gift_aid\Event\GiftAidEvents;
use Drupal\gift_aid\Event\GiftAidRelatedContextsEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 *
 */
class GiftAidRelatedContextsSubscriber implements EventSubscriberInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new EventSubscriber.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [
      GiftAidEvents::RELATED_CONTEXTS => ['addContexts'],
    ];
    return $events;
  }

  /**
   * Relate order and user contexts.
   *
   * @param \Drupal\gift_aid\Event\GiftAidRelatedContextsEvent $event
   *   The related contexts event.
   */
  public function addContexts(GiftAidRelatedContextsEvent $event) {
    $orderStorage = $this->entityTypeManager->getStorage('commerce_order');
    $userStorage = $this->entityTypeManager->getStorage('user');

    // Load users from orders before orders from users,
    // because orders:user is many:one.
    $orders = $event->getContexts()['commerce_order'] ?? [];
    if (!empty($orders)) {
      // The user may not have existed before the order.
      $event->addCacheTags($userStorage->getEntityType()->getListCacheTags());
      $orders = $orderStorage->loadMultiple($orders);
      foreach ($orders as $order) {
        $customer = $order->getCustomer();
        $event->addCacheTags($order->getCacheTags());
        if (!empty($customer) && !$customer->isAnonymous()) {
          $event->addContext($customer);
        }
      }
    }

    $users = $event->getContexts()['user'] ?? [];
    if (!empty($users)) {
      $orderStorage = $this->entityTypeManager->getStorage('commerce_order');
      $orderIds = $orderStorage
        ->getQuery()
        ->accessCheck(FALSE)
        ->condition('uid', $users, 'IN')
        ->condition('state', 'draft', '<>')
        ->execute();
      $event->addContextIds('commerce_order', $orderIds);
      $event->addCacheTags($orderStorage->getEntityType()->getListCacheTags());
    }
  }

}
