<?php

namespace Drupal\gift_aid_commerce;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\gift_aid\Entity\Declaration;
use Drupal\gift_aid\Entity\DeclarationInterface;

/**
 *
 */
class OrderService implements OrderServiceInterface {

  /**
   * Constructs a new OrderService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(protected readonly EntityTypeManagerInterface $entityTypeManager) {
  }

  /**
   * {@inheritdoc}
   */
  public function declare(OrderInterface $order) {
    $evidence_storage = $this->entityTypeManager->getStorage('gift_aid_evidence');
    $declaration_storage = $this->entityTypeManager->getStorage('gift_aid_declaration');
    $donor_storage = $this->entityTypeManager->getStorage('gift_aid_donor');
    $customer = $order->getCustomer();
    $context = $customer->isAnonymous() ? $order : $customer;

    // Only create a declaration if one does not already exist.
    if (!empty($declaration_storage->loadByContext($context))) {
      return;
    }

    $donor = $donor_storage->loadOrCreateByContext($order);
    if ($donor->isNew()) {
      if (($profile = $order->getBillingProfile()) && ($address = $profile->get('address')->first())) {
        $donor->setAddress($address)->save();
      }
      else {
        \Drupal::logger('gift_aid_commerce')->info("Cannot collect Gift Aid declaration for order " . $order->id() . " as address is not specified.");
        return;
      }
    }

    $evidence = $evidence_storage->create([
      'type' => 'commerce_order',
      'field_order' => $order,
    ]);
    $evidence->save();

    $declaration_storage->create()
      ->setCharityId($order->getData('gift_aid_commerce_charity_id'))
      ->setDonor($donor)
      ->setEvidence($evidence)
      ->setExplanation($order->getData('gift_aid_commerce_explanation'))
      ->setClaimableDate(Declaration::today())
      ->setValidity(DeclarationInterface::DECLARATION_VALID)
      ->save();
  }

}
