<?php

namespace Drupal\gift_aid\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\user\UserInterface;

/**
 * Gift aid tab for user context.
 */
class ContextOverviewController extends ControllerBase {

  /**
   * Create an overview page for a user context.
   *
   * @return array
   *   A render array.
   */
  public function view(UserInterface $user) {
    // Load the donor entity from the user.
    $storage = \Drupal::entityTypeManager()->getStorage('gift_aid_donor');
    $donor = $storage->loadOrCreateByContext($user);

    if (!$donor->isNew()) {
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder('gift_aid_donor');
      $build['donor'] = $view_builder->view($donor, 'account');
    }
    else {
      $form_builder = \Drupal::service('entity.form_builder');
      $build['donor'] = $form_builder->getForm($donor, 'account');
    }

    return $build;
  }

  /**
   * @todo Not used - delete.
   */
  protected function buildOngoing(array &$build, UserInterface $user) {
    // We don't have access to cacheability metadata for context relationships,
    // so we can't guarantee that the set of contexts related to the current context
    // have not changed, and therefore that the set of declarations is unchanged.
    // @todo BUT NOW WE DO !!!!!!!!!!!!
    $build['#cache']['max-age'] = 0;

    // ALL OF THIS ONLY WORKS IF THERE IS A DEFAULT CHARITY!!
    $storage = $this->entityTypeManager()->getStorage('gift_aid_declaration');
    $ongoing = $storage->loadOngoingByContext($user);
    if (empty($ongoing)) {
      $build['message'] = [
        '#markup' => '<p>No Gift Aid currently.</p>',
      ];
      $link_options = ['attributes' => ['class' => ['button']]];
      if ($is_related) {
        // BUTTON FOR MAKE DECLARATION.
      }
      else {
        $url = Url::fromRoute('entity.gift_aid_declaration.add_form', [
          'context-type' => $context->getEntityTypeId(),
          'context-id' => $context->id(),
        ])->setOptions($link_options);
        $link = Link::fromTextAndUrl($this->t('Add'), $url);
        $build['add'] = $link->toRenderable();
        $build['add']['#access'] = $url->access();
      }
    }
    else {
      // All ongoing declarations are forced to have the same end date in Declaration::postSave().
      $declaration = reset($ongoing);
      if ($declaration->hasEndDate()) {
        $build['message'] = [
          '#markup' => '<p>Ends on ' . $declaration->getEndDate(TRUE) . ' .</p>',
        ];
        $link_options = ['attributes' => ['class' => ['button']]];
        $url = Url::fromRoute('entity.gift_aid_declaration.edit_form', [
          'gift_aid_declaration' => $declaration->id(),
        ])->setOptions($link_options);
        $link = Link::fromTextAndUrl($this->t('Extend'), $url);
        $build['extend'] = $link->toRenderable();
        $build['extend']['#access'] = $url->access();
      }
      else {
        $build['message'] = [
          '#markup' => '<p>Indefinite .</p>',
        ];
      }

      // Ongoing declarations can be cancelled as their end date is either unspecified or in the future.
      $link_options = ['attributes' => ['class' => ['button']]];
      $url = Url::fromRoute('entity.gift_aid_declaration.edit_form', [
        'gift_aid_declaration' => $declaration->id(),
        'mode' => 'cancel',
      ])->setOptions($link_options);
      $link = Link::fromTextAndUrl($this->t('Cancel'), $url);
      $build['cancel'] = $link->toRenderable();
      $build['cancel']['#access'] = $url->access();
    }
  }

  /**
   * Checks access for the Gift Aid account form.
   *
   * @param \Drupal\user\UserInterface $user
   *   The account to use in the form.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   An access result object.
   */
  public function checkAccess(UserInterface $user) {
    $account = $this->currentUser();

    // Deny access for anonymous user.
    if ($user->isAnonymous()) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowedIfHasPermission($account, 'record gift aid declarations made by others')
      ->orIf(AccessResult::allowedIfHasPermission($account, 'make gift aid declaration')
        ->andIf(AccessResult::allowedIf($user->id() == $account->id())));
  }

}
