<?php

namespace Drupal\gift_aid\Declaration;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Link;

/**
 * Defines a class to build a listing of declarations.
 *
 * @ingroup gift_aid
 */
class DeclarationListBuilder extends EntityListBuilder {

  /**
   * The entities to include in the listing.
   */
  protected ?array $entities = NULL;

  /**
   * Sets the entities to include in the listing.
   *
   * @param array $entities
   *   The entities.
   */
  public function setEntities(?array $entities) {
    $this->entities = $entities;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('ID');
    $header['evidence_type'] = $this->t('Evidence Type');
    $header['charity'] = $this->t('Charity');
    if (!$this->entities) {
      $header['donor'] = $this->t('Donor');
    }
    $header['status'] = $this->t('Status');
    $header['declared_date'] = $this->t('Declared Date');
    $header['start_date'] = $this->t('Start Date');
    $header['end_date'] = $this->t('End Date');

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\gift_aid\Entity\Declaration $entity */
    $row['id'] = Link::createFromRoute(
      $entity->id(),
      'entity.gift_aid_declaration.canonical',
      ['gift_aid_declaration' => $entity->id()]
    );
    $row['evidence_type'] = $entity->getEvidence()->type->entity->toLink(NULL, 'edit-form')->toString();
    $row['charity'] = $entity->getCharity()->toLink(NULL, 'edit-form')->toString();
    if (!$this->entities) {
      $row['donor'] = $entity->getDonor()->toLink()->toString();
    }
    $row['status'] = $entity->getStatusString();
    $row['declared_date'] = $entity->getDeclaredDate(TRUE);
    $row['start_date'] = $entity->getStartDate(TRUE) ?? '-';
    $row['end_date'] = $entity->getEndDate(TRUE) ?? '-';

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function load() {
    return $this->entities ?? parent::load();
  }

}
