<?php

namespace Drupal\gift_aid\Declaration;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\gift_aid\Entity\CharityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the gift aid declaration storage.
 */
class DeclarationStorage extends SqlContentEntityStorage implements DeclarationStorageInterface {

  /**
   * @var \Drupal\gift_aid\GiftAidRelatedContextsManagerInterface
   */
  protected $relatedContextsManager;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    $instance = parent::createInstance($container, $entity_type);
    $instance->relatedContextsManager = $container->get('gift_aid.related_contexts_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function loadByContext(EntityInterface $context, ?CharityInterface $charity = NULL) {
    $properties = [
      'donor.entity:gift_aid_donor.context.target_type' => $context->getEntityTypeId(),
      'donor.entity:gift_aid_donor.context.target_id' => $context->id(),
    ];

    if ($charity) {
      $properties['charity.target_id'] = $charity->id();
    }

    return $this->loadByProperties($properties);
  }

  /* @todo Remainder not used - delete. */

  /**
   * {@inheritdoc}
   */
  public function loadByContexts(array $contexts, ?CharityInterface $charity = NULL) {
    $query = $this->getQuery();
    $query->accessCheck(FALSE);

    $orGroup = $query->orConditionGroup();
    foreach ($contexts as $entity_type => $entity_ids) {
      if (!empty($entity_ids)) {
        $andGroup = $query->andConditionGroup();
        $andGroup->condition('donor.entity:gift_aid_donor.context.target_type', $entity_type);
        $andGroup->condition('donor.entity:gift_aid_donor.context.target_id', array_values($entity_ids), 'IN');
        $orGroup->condition($andGroup);
      }
    }
    $query->condition($orGroup);

    if ($charity) {
      $query->condition('charity.target_id', $charity->id());
    }

    $result = $query->execute();
    return $result ? $this->loadMultiple($result) : [];
  }

  /**
   * {@inheritdoc}
   */
  public function loadByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL) {
    $contexts = $this->relatedContextsManager->getRelatedContexts($original_context);
    return $this->loadByContexts($contexts, $charity);
  }

  /**
   * {@inheritdoc}
   */
  public function loadOngoingByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL) {
    $declarations = $this->loadByAnyContext($original_context, $charity);
    return array_filter($declarations, function ($declaration) {
      return $declaration->isOngoing();
    });
  }

  /**
   * {@inheritdoc}
   */
  public function getActiveByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL) {
    $declarations = $this->loadByAnyContext($original_context, $charity);
    $active = NULL;
    $later = TRUE;
    foreach ($declarations as $declaration) {
      if ($active) {
        // Get the one with no end date or the end date furthest in the future.
        $later = $active->hasEndDate() && (!$declaration->hasEndDate() || ($declaration->getEndDate(TRUE) > $active->getEndDate(TRUE)));
      }
      if ($declaration->isOngoing() && $later) {
        $active = $declaration;
      }
    }
    return $active;
  }

  /**
   * {@inheritdoc}
   */
  public function hasActiveByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL) {
    return (bool) $this->loadOngoingByAnyContext($original_context, $charity);
  }

}
