<?php

namespace Drupal\gift_aid\Donor;

use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Url;

/**
 * View builder handler for the Donor entity.
 */
class DonorViewBuilder extends EntityViewBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildComponents(array &$build, array $entities, array $displays, $view_mode) {
    parent::buildComponents($build, $entities, $displays, $view_mode);

    $current_user = \Drupal::currentUser();
    $entity_type_manager = \Drupal::entityTypeManager();
    $builder = $entity_type_manager->getListBuilder('gift_aid_declaration');

    foreach ($entities as $id => $donor) {
      $display = $displays['gift_aid_donor'];
      $actions = [];

      if ($donor->id()) {
        $actions['declare'] = [
          '#theme' => 'menu_local_action',
          '#link' => [
            'title' => $this->t('Add Declaration'),
            'url' => Url::fromRoute('entity.gift_aid_declaration.add_form', [], ['query' => ['donor' => $donor->id()]]),
          ],
          '#access' => $current_user->hasPermission('record gift aid declarations made by others'),
        ];

        $actions['edit'] = [
          '#theme' => 'menu_local_action',
          '#link' => [
            'title' => $this->t('Edit'),
            'url' => Url::fromRoute('entity.gift_aid_donor.edit_form', ['gift_aid_donor' => $donor->id()]),
          ],
          '#access' => $donor->access('update'),
        ];
      }

      if ($actions) {
        $build[$id]['actions'] = [
          '#prefix' => '<ul class="action-links">',
          '#suffix' => '</ul>',
          '#weight' => -10,
        ] + $actions;
      }

      if ($donor->id()) {
        $declarations = \Drupal::entityTypeManager()
          ->getStorage('gift_aid_declaration')
          ->loadByProperties(['donor' => $donor->id()]);
      }
      else {
        $declarations = [];
      }

      foreach ($declarations as $declaration) {
        if ($declaration->isOngoing()) {
          $ongoing = TRUE;
          if (!$declaration->hasEndDate()) {
            $endless = TRUE;
          }
        }
      }

      if ($display->getComponent('status')) {
        $status = isset($endless) ? $this->t("Valid") : (isset($ongoing) ? $this->t("Limited") : ($declarations ? $this->t("Not currently valid") : $this->t("None")));
        $build[$id]['status'] = [
          '#type' => 'item',
          '#title' => $this->t('Status'),
          '#markup' => $status,
        ];
      }

      if ($display->getComponent('declarations')) {
        $builder->setEntities($declarations);
        $build[$id]['declarations'] = [
          '#type' => 'details',
          '#title' => $this->t('Declarations'),
          '#open' => TRUE,
          'list' => $builder->render(),
          '#access' => \Drupal::currentUser()->hasPermission('record gift aid declarations made by others'),
        ];
      }
    }
  }

}
