<?php

namespace Drupal\gift_aid\Form\Declaration;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for Declaration edit forms.
 *
 * @ingroup gift_aid
 */
class DeclarationForm extends ContentEntityForm {

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->messenger = $container->get('messenger');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareEntity() {
    parent::prepareEntity();
    $this->entity->setRevisionLogMessage(NULL);

    // Allow donor and charity to be prepopulated by querystring when creating declarations.
    if ($this->entity->isNew()) {
      $donor_id = \Drupal::request()->query->get('donor');
      if ($donor_id) {
        $this->entity->setDonorId($donor_id);
      }

      $charity_id = \Drupal::request()->query->get('charity');
      if ($charity_id) {
        $this->entity->setCharityId($charity_id);
      }

    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = parent::buildEntity($form, $form_state);
    $entity->setNewRevision();
    $entity->setRevisionUserId($this->currentUser()->id());
    $entity->setRevisionCreationTime($this->time->getRequestTime());

    return $entity;
  }

  // @todo VALIDATION: only admins should be able to backdate end date,
  // except when creating.
  // Button text should vary depending on whether declaration is ongoing.

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    // Hide context and charity if set by querystring, but only when creating.
    if ($this->entity->isNew()) {
      if (\Drupal::request()->query->get('donor')) {
        $form['donor']['#access'] = FALSE;
      }
      if (\Drupal::request()->query->get('charity')) {
        $form['charity']['#access'] = FALSE;
      }
    }
    // Show/hide Start and End dates depending on the DateBased state.
    // @todo
    /*    $date_based_checked = [
    ':input[name="date-based[value]"]' => ['checked' => TRUE],
    ];
    $form['start_date']['#states'] = [
    'visible' => $date_based_checked,
    ];
    $form['start_date']['widget'][0]['value']['#states'] = [
    'required' => $date_based_checked,
    ];
    $form['end_date']['#states'] = [
    'visible' => $date_based_checked,
    ];*/

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $entity->setNewRevision();

    $status = parent::save($form, $form_state);

    switch ($status) {
      case SAVED_NEW:
        $this->messenger->addMessage($this->t('Recorded a @bundle Gift Aid declaration for %label.', [
          '@bundle' => $entity->bundle(),
          '%label' => $entity->getDonor()->label(),
        ]));
        break;

      default:
        $this->messenger->addMessage($this->t('Updated the @bundle Gift Aid declaration for %label.', [
          '@bundle' => $entity->bundle(),
          '%label' => $entity->getDonor()->label(),
        ]));
    }
    $form_state->setRedirect('entity.gift_aid_declaration.canonical', ['gift_aid_declaration' => $entity->id()]);
    return $status;
  }

}
