<?php

namespace Drupal\gift_aid\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gift_aid\Entity\Declaration;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for showing an overview of declarations in a context.
 *
 * @ingroup gift_aid
 * @todo Not used - delete.
 */
class DeclarationContextOverviewForm extends FormBase {

  /**
   * The date entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The declaration context entity.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $context;

  /**
   * Construct a DeclarationCancelForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Component\Datetime\TimeInterface $time
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, TimeInterface $time) {
    $this->entityTypeManager = $entity_type_manager;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return "gift_aid_declaration_cancel_form";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // @todo Fix this code.
    $declaration_storage = $this->entityTypeManager->getStorage('gift_aid_declaration');
    $declarations =

    $date_format = $this->entityTypeManager->getStorage('date_format')->load('html_date')->getPattern();
    $form['end_date'] += [
      '#date_date_format' => $date_format,
      '#date_date_element' => 'date',
      '#date_time_type' => 'none',
      '#default_value' => Declaration::today(),
      '#required' => TRUE,
      '#title' => $this->t("End date"),
      '#decription' => $this->t("The last date that Gift Aid should be claimed. This cannot be in the past."),
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#name' => 'cancel',
      '#value' => $this->t('Cancel'),
    ];

    return $form;
  }

  /**
   *
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $date = $form_state->getValue('end_date');
    if (empty($date) || !($date instanceof DrupalDateTime)) {
      // @todo Fix commented out code.
      // $form_state->setErrorByName('', "An end date must be provided.");
    }

    if ($date < Declaration::today()) {
      $form_state->setErrorByName('', "The end date must not be in the past.");
    }

    parent::validateForm($form, $form_state);
  }

  /**
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $date = $form_state->getValue('end_date');
    $storage = $this->entityTypeManager->getStorage('gift_aid_declaration');
    $declarations = $storage->loadByAnyContext($this->getContext());
    foreach ($declarations as $declaration) {
      if ($declaration->isDateBased()) {
        if (!$declaration->hasEndDate() || $declaration->getEndDate(TRUE) > $date) {
          $declaration->setEndDate($date)->save();
        }
      }
    }

  }

  /**
   * Get the declaration context entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The context entity.
   */
  protected function getContext() {
    if (!$this->context) {
      $entity_type = $this->getRouteMatch()->getParameter('entity_type');
      $entity_id = $this->getRouteMatch()->getParameter('entity_type');
      $storage = $this->entityTypeManager->getStorage($entity_type);
      $this->context = $storage->load($entity_id);
    }
    return $this->context;
  }

  /**
   * Route title callback.
   *
   * @return string
   *   The title for the DeclarationCancelForm.
   */
  public function getTitle() {
    return $this->t('Cancel Gift Aid declarations for %label', ['%label' => $this->getContext()->label()]);
  }

}
