<?php

namespace Drupal\gift_aid\Form\Donor;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gift_aid\Entity\Declaration;
use Drupal\gift_aid\Entity\DeclarationInterface;

/**
 * Form controller for Donor edit on the user account tab.
 *
 * @ingroup gift_aid
 */
class DonorAccountForm extends DonorForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $context = $this->entity->getContext();
    $self = ($context->getEntityTypeId() == 'user') && ($context->id() == $this->currentUser()->id());

    if ($self) {
      // @todo Create common code with GiftAidDeclaration.php.
      $form['introduction'] = [
        '#markup' => $this->t("Increase your donation at no extra cost to you."),
      ];
      $form['explanation'] = [
        '#markup' => $this->t("I want to Gift Aid my donation and any donations I make in the future or have made in the past four years to {{ charity.label }}. I am a UK taxpayer and understand that if I pay less Income Tax and/or Capital Gains Tax than the amount of Gift Aid claimed on all my donations in that tax year it is my responsibility to pay any difference."),
      ];
    }
    else {
      $form['status'] = [
        '#title' => $this->t('Status'),
        '#markup' => $this->t('Gift Aid is not currently active. Enter an address to activate it.'),
      ];
    }

    $form = parent::form($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $donor = $this->entity;
    $status = ContentEntityForm::save($form, $form_state);

    if (isset($form['explanation'])) {
      $entity_type_manager = \Drupal::entityTypeManager();
      $evidence = $entity_type_manager->getStorage('gift_aid_evidence')->create(['type' => 'user']);
      $evidence->save();

      $declaration = $entity_type_manager->getStorage('gift_aid_declaration')->create()
        ->setDonor($donor)
        ->setEvidence($evidence)
        ->setExplanation($form['explanation']['#markup'])
        ->setClaimableDate(Declaration::today())
        ->setValidity(DeclarationInterface::DECLARATION_VALID);
      $declaration->save();

      $this->messenger()->addStatus($this->t('Thank you for using Gift Aid.'));
    }
    else {
      $this->messenger()->addStatus($this->t('Gift Aid enabled for %label.', ['%label' => $donor->label()]));
    }
    return $status;
  }

}
